@@ -1,7 +1,7 @@
 #!/usr/bin/python
 #
 # Urwid tutorial documentation generation program
-#    Copyright (C) 2004-2006  Ian Ward
+#    Copyright (C) 2004-2007  Ian Ward
 #
 #    This library is free software; you can redistribute it and/or
 #    modify it under the terms of the GNU Lesser General Public
@@ -31,9 +31,6 @@ try:
 except:
 	templayer = None
 
-try: True # old python?
-except: False, True = 0, 1
-
 examples = {}
 
 interp_line = "#!/usr/bin/python\n\n"
@@ -47,7 +44,7 @@ def example_min():
 	ui = urwid.curses_display.Screen()
 
 	def run():
-		canvas = urwid.Canvas( ["Hello World"] )
+		canvas = urwid.TextCanvas(["Hello World"])
 		ui.draw_screen( (20, 1), canvas )
 		
 		while not ui.get_input():
@@ -177,10 +174,11 @@ def example_frlb():
 	import urwid.curses_display
 	import urwid
 
-	class Conversation:
+	class Conversation(object):
 		def __init__(self):
-			self.items = [ self.new_question() ]
-			self.listbox = urwid.ListBox( self.items )
+			self.items = urwid.SimpleListWalker(
+				[self.new_question()])
+			self.listbox = urwid.ListBox(self.items)
 			instruct = urwid.Text("Press F8 to exit.")
 			header = urwid.AttrWrap( instruct, 'header' )
 			self.top = urwid.Frame(self.listbox, header)
@@ -227,10 +225,11 @@ def example_lbcont():
 	import urwid.curses_display
 	import urwid
 
-	class Conversation:
+	class Conversation(object):
 		def __init__(self):
-			self.items = [ self.new_question() ]
-			self.listbox = urwid.ListBox( self.items )
+			self.items = urwid.SimpleListWalker(
+				[self.new_question()])
+			self.listbox = urwid.ListBox(self.items)
 			instruct = urwid.Text("Press F8 to exit.")
 			header = urwid.AttrWrap( instruct, 'header' )
 			self.top = urwid.Frame(self.listbox, header)
@@ -304,7 +303,7 @@ def example_lbscr():
 		urwid.Text("The end."),
 	] ]
 	# CUT HERE
-	class RevealFocus:
+	class RevealFocus(object):
 		def __init__(self):
 			self.listbox = urwid.ListBox( CONTENT )
 			self.head = urwid.Text("Pressed:")
@@ -368,14 +367,15 @@ def example_wanat():
 			return 1
 		def render( self, (maxcol,), focus=False ):
 			num_pudding = maxcol / len("Pudding")
-			return urwid.Canvas( ["Pudding"*num_pudding] )
+			return urwid.TextCanvas(["Pudding"*num_pudding]) 
 
 	class BoxPudding( urwid.BoxWidget ):
 		def selectable( self ):
 			return False
 		def render( self, (maxcol, maxrow), focus=False ):
 			num_pudding = maxcol / len("Pudding")
-			return urwid.Canvas( ["Pudding"*num_pudding] * maxrow )
+			return urwid.TextCanvas(
+				["Pudding"*num_pudding] * maxrow)
 
 def example_wanat_new():
 	class NewPudding( urwid.FlowWidget ):
@@ -392,7 +392,7 @@ def example_wanat_new():
 			return urwid.Text( "Pudding"*num_pudding )
 
 def example_wanat_multi():
-	class MultiPudding:
+	class MultiPudding( urwid.Widget ):
 		def selectable( self ):
 			return False
 		def rows( self, (maxcol,), focus=False ):
@@ -404,7 +404,8 @@ def example_wanat_multi():
 			else:
 				(maxcol, maxrow) = size
 			num_pudding = maxcol / len("Pudding")
-			return urwid.Canvas( ["Pudding"*num_pudding] * maxrow )
+			return urwid.TextCanvas(
+				["Pudding"*num_pudding] * maxrow )
 
 examples["wsel"] = ["example_wsel"]
 def example_wsel():
@@ -420,7 +421,7 @@ def example_wsel():
 			pudding = self.pudding
 			if focus: 
 				pudding = pudding.upper()
-			return urwid.Canvas( [pudding*num_pudding] )
+			return urwid.TextCanvas( [pudding*num_pudding] )
 		def keypress( self, (maxcol,), key ):
 			if len(key)>1:
 				return key
@@ -447,7 +448,8 @@ def example_wcur():
 			cursor = None
 			if focus:
 				cursor = self.get_cursor_coords((maxcol,))
-			return urwid.Canvas( ["Pudding"*num_pudding], [], cursor )
+			return urwid.TextCanvas(
+				["Pudding"*num_pudding], [], cursor )
 		def get_cursor_coords( self, (maxcol,) ):
 			col = min(self.cursor_col, maxcol-1)
 			return col, 0
