@@ -106,6 +106,7 @@ class CanvasCache(object):
 	def cleanup(cls, ref):
 		cls.cleanups += 1
 		w = cls._refs.get(ref, None)
+		del cls._refs[ref]
 		if not w:
 			return
 		widget, size, focus = w
@@ -119,9 +120,9 @@ class CanvasCache(object):
 		if not sizes:
 			try:
 				del cls._widgets[widget]
+				del cls._deps[widget]
 			except KeyError:
 				pass
-		del cls._refs[ref]
 	cleanup = classmethod(cleanup)
 
 	def clear(cls):
@@ -685,12 +686,17 @@ class CompositeCanvas(Canvas):
 		intact."""
 		if self.widget_info:
 			raise self._finalized_error
-		
-		for num_rows, cviews in self.shards:
-			for i in range(len(cviews)):
-				cv = cviews[i]
+
+		shards = []
+		for num_rows, original_cviews in self.shards:
+			new_cviews = []
+			for cv in original_cviews:
 				if cv[4] is None:
-					cviews[i] = cv[:4] + (a,) + cv[5:]
+					new_cviews.append(cv[:4] + (a,) + cv[5:])
+				else:
+					new_cviews.append(cv)
+			shards.append((num_rows, new_cviews))
+		self.shards = shards
 
 	def set_depends(self, widget_list):
 		"""
