@@ -718,6 +718,8 @@ class IntEdit(Edit):
 			return 0
 
 class WidgetWrap(Widget):
+	no_cache = ["rows"]
+
 	def __init__(self, w):
 		"""
 		w -- widget to wrap, stored as self.w
@@ -1022,8 +1024,10 @@ class Button(WidgetWrap):
 		if button != 1 or not is_mouse_press(event):
 			return False
 			
-		self.on_press( self )
-		return True
+		if self.on_press:
+			self.on_press( self )
+			return True
+		return False
 	
 
 
@@ -1405,7 +1409,7 @@ class Filler(BoxWidget):
 		self.__super.__init__()
 		vt,va,ht,ha=decompose_valign_height(valign,height,FillerError)
 		
-		self.body = body
+		self._body = body
 		self.valign_type, self.valign_amount = vt, va
 		self.height_type, self.height_amount = ht, ha
 		if self.height_type not in ('fixed', None):
@@ -1413,9 +1417,16 @@ class Filler(BoxWidget):
 		else:
 			self.min_height = None
 	
+	def get_body(self):
+		return self._body
+	def set_body(self, body):
+		self._body = body
+		self._invalidate()
+	body = property(get_body, set_body)
+	
 	def selectable(self):
 		"""Return selectable from body."""
-		return self.body.selectable()
+		return self._body.selectable()
 	
 	def filler_values(self, (maxcol, maxrow), focus):
 		"""Return the number of rows to pad on the top and bottom.
@@ -2160,7 +2171,7 @@ class Pile(Widget): # either FlowWidget or BoxWidget
 		if focus_item is None:
 			focus_item = 0
 		self.set_focus(focus_item)
-		self.pref_col = None
+		self.pref_col = 0
 
 	def selectable(self):
 		"""Return True if the focus item is selectable."""
